@@ -11,11 +11,12 @@ use std::{
 use device::FutureDevice;
 use futures::Future;
 use reactor::Reactor;
+pub use smoltcp;
 use smoltcp::{
     iface::{EthernetInterfaceBuilder, NeighborCache, Routes},
-    wire::{EthernetAddress, IpAddress, IpCidr},
+    wire::{EthernetAddress, IpAddress, IpCidr, IpProtocol, IpVersion},
 };
-pub use socket::{TcpListener, TcpSocket, UdpSocket};
+pub use socket::{RawSocket, TcpListener, TcpSocket, UdpSocket};
 pub use socket_alloctor::BufferSize;
 use tokio::sync::Notify;
 
@@ -28,7 +29,7 @@ pub mod util;
 pub struct NetConfig {
     pub ethernet_addr: EthernetAddress,
     pub ip_addr: IpCidr,
-    pub gateway: IpAddress,
+    pub gateway: Vec<IpAddress>,
     pub buffer_size: BufferSize,
 }
 
@@ -45,10 +46,17 @@ impl Net {
         config: NetConfig,
     ) -> (Net, impl Future<Output = ()> + Send) {
         let mut routes = Routes::new(BTreeMap::new());
-        match config.gateway {
-            IpAddress::Ipv4(v4) => routes.add_default_ipv4_route(v4).unwrap(),
-            _ => panic!("gateway should be set"),
-        };
+        for gateway in config.gateway {
+            match gateway {
+                IpAddress::Ipv4(v4) => {
+                    routes.add_default_ipv4_route(v4).unwrap();
+                }
+                IpAddress::Ipv6(v6) => {
+                    routes.add_default_ipv6_route(v6).unwrap();
+                }
+                _ => panic!("Unsupported address"),
+            };
+        }
         let neighbor_cache = NeighborCache::new(BTreeMap::new());
         let interf = EthernetInterfaceBuilder::new(device)
             .ethernet_addr(config.ethernet_addr)
@@ -92,6 +100,13 @@ impl Net {
         let addr = self.set_address(addr);
         UdpSocket::new(self.reactor.clone(), addr.into()).await
     }
+    pub async fn raw_socket(
+        &self,
+        ip_version: IpVersion,
+        ip_protocol: IpProtocol,
+    ) -> io::Result<RawSocket> {
+        RawSocket::new(self.reactor.clone(), ip_version, ip_protocol).await
+    }
     fn set_address(&self, mut addr: SocketAddr) -> SocketAddr {
         if addr.ip().is_unspecified() {
             addr.set_ip(match self.ip_addr.address() {
